@@ -24,6 +24,8 @@ import numpy.matlib as npmt
 import copy
 from .ola import *
 from .praattools import *
+from .transformtools import (transform_formant, transform_formant_from_dist,
+                             sparse_transform)
 
 class SpeechAudio:
 
@@ -740,7 +742,7 @@ class SpeechAudio:
 
             return obj_out
 
-    def alignread(self, fileName):
+    def alignread(self, fileName, utt_id=None):
 
         if fileName.lower().find('.textgrid') > 0:
             labels, start, stop = func_readTextgrid(fileName)
@@ -770,7 +772,10 @@ class SpeechAudio:
         elif fileName.lower().find('.whc') > 0:
             labels, instants = read_whc_segmentation_files(fileName)
         elif fileName.lower().find('.seg') > 0:
-            labels, instants = read_seg_segmentation_files(fileName)
+            if utt_id is None:
+                labels, instants = read_seg_segmentation_files(fileName)
+            else:
+                labels, instants = read_seg_segmentation_files_2(fileName, utt_id)
         else:
             raise FormatError('Unknown file format')
 
@@ -778,6 +783,29 @@ class SpeechAudio:
         self.phonetic_instants = instants
 
         return labels, instants
+    
+    def sparse_transform(self, W_nominal, W_mask, nwin=240, novlp=None):
+
+        novlp, nfft, nhop = ola_param(nwin, novlp)
+        x2, x_out = init_ola(self.signal, nwin, nhop)
+        
+        Spx2 = SpeechAudio("signal", x2, 
+                           "sampling_frequency", self.sampling_frequency)
+        Sxx, fxx, txx = Spx2.computecepstrumenvelope(nwin, novlp)
+        Sxx_target = sparse_transform(Sxx, W_nominal, W_mask)
+        k = 0
+        isC = True
+        while isC:
+            xwin, fft_xwin, isC, idx = spec_frame(x2, k, nwin, nhop, nfft)
+            h_orig = np.abs(Sxx[:, k])
+            h_new = np.abs(Sxx_target[:, k])
+            x_out = change_frame(fft_xwin, h_new, h_orig,
+                                         nfft, idx, x_out, xwin=xwin, isNorm=False)
+    
+            k, isC = check_frame(k, isC, maxFrame=Sxx.shape[1])
+        x_out = x_out[nhop:-nhop] / 1.5
+        x_out[np.isnan(x_out)] = 0
+        self.signal = x_out
 
     def cepstraltransform(self, transOper, nwin=1024,
                           novlp=None, sr=8000, freq_out=None,
@@ -858,7 +886,7 @@ class SpeechAudio:
             xwin, fft_xwin, isC, idx = spec_frame(x2, k, nwin, nhop, nfft)
 
             x_out = change_frame(fft_xwin, Sxx_trans[:,k], Sxx_in[:,k], nfft, idx, x_out,
-                                     xwin, isChange=True)
+                                     xwin, isChange=True, isNorm=False)
             k, isC = check_frame(k, isC, maxFrame=Sxx_in.shape[1])
         x_out = x_out[nhop:-nhop] / 1.5
 
@@ -899,9 +927,9 @@ class SpeechAudio:
             segStart = []
             segStop = []
             events = []
-            for kE in range(nEvents):                
-                segStart = inst[kE, 0]
-                segStop = inst[kE, 1]
+            for kE in range(nEvents):
+                segStart = "%.3f" %(inst[kE, 0])
+                segStop = "%.3f" %(inst[kE, 1])
                 events.append({'segment_name': lbl[kE],
                                'segment_start': segStart,
                                'segment_stop': segStop})
@@ -1015,23 +1043,23 @@ class SpeechAudio:
 
         self.signal = x_out
         self.formants = formTarget
-
-    def frequency_warping(self, form_output, nwin=512, novlp=None, tf_add=None):
+       
+    def frequency_warping(self, dil_fact, trans_fact, dist=None, ord_filt=None,
+                      nwin=512, novlp=None):
 
         novlp, nfft, nhop = ola_param(nwin, novlp)
-
-        if tf_add is not None:
-            tf_add.interpolate(fxx)
-            tf_resp = tf_add.values
-        else:
-            tf_resp = 1
-
         x2, x_out = init_ola(self.signal, nwin, nhop)
+        
         Spx2 = SpeechAudio('signal', x2, 
                            'sampling_frequency', self.sampling_frequency)
         Sxx, fxx, txx = Spx2.computecepstrumenvelope(nwin, novlp)
-        Fr = Spx2.formantestimate(nwin, novlp, ord_filt=None)
-        Fr.interpolate(txx)
+        form_in = Spx2.formantestimate(nwin, novlp, ord_filt=ord_filt)
+        form_in.interpolate(txx)
+
+        if dist is None:
+            form_output = transform_formant(form_in, dil_fact, trans_fact)
+        else:
+            form_output = transform_formant_from_dist(form_in, dist)
         form_output.interpolate(txx)
         
         k = 0
@@ -1039,18 +1067,17 @@ class SpeechAudio:
         while isC:
             xwin, fft_xwin, isC, idx = spec_frame(x2, k, nwin, nhop, nfft)
             h_orig = np.abs(Sxx[:, k])
-            if (Fr.frequency[:,k] != 0).all():
-                new_freq = ceps2peaks(h_orig, Fr.frequency[:,k], fxx)
-                h_new = dfw(h_orig, fxx,
-                        np.sort(form_output.frequency[:,k]), new_freq) * tf_resp
-
+            if (form_in.frequency[:,k] != 0).all():
+                # new_freq = ceps2peaks(h_orig, form_in.frequency[:,k], fxx)
+                new_freq = form_in.frequency[:,k]
+                h_new = dfw(h_orig, fxx, np.sort(form_output.frequency[:,k]), new_freq)
             else:
                 h_new = 1
-
+    
             x_out = change_frame(fft_xwin, h_new, h_orig,
                                          nfft, idx, x_out, xwin=xwin,
-                                         isChange=(Fr.frequency[:,k] != 0).all())
-
+                                         isChange=(form_in.frequency[:,k] != 0).all())
+    
             k, isC = check_frame(k, isC, maxFrame=Sxx.shape[1])
         x_out = x_out[nhop:-nhop] / 1.5
         x_out[np.isnan(x_out)] = 0
@@ -1202,7 +1229,66 @@ class SpeechAudio:
         else:
             print('Warning : The output sampling frequency '
                   'is the same as the input. Nothing''s done!')
+            
+    def computeVQ(self, nwin=1024, novlp=None, nfft=None, meth="rms"):
 
+        if self.voiced is None or self.unvoiced is None:
+            raise ValueError("No separation has been done. Please run X-GLOS")
+
+        sr = self.sampling_frequency;
+        tvec = np.arange(0, len(self.signal)) / sr
+        
+        if novlp is None:
+            novlp = int(nwin * 3 / 4)
+        if nfft is None: 
+            nfft = 2**nextpow2(sr)
+        
+        V = self.voiced
+        U = self.unvoiced
+        
+        if meth.lower() == "spectro":
+            Sxx_V, fxx, txx = V.computespectrogram(nwin, novlp=novlp, nfft=nfft)
+            Sxx_U, _, _ = U.computespectrogram(nwin, novlp=novlp, nfft=nfft)
+            
+            Den = np.sum(abs(Sxx_U)**2, 0) + np.sum(abs(Sxx_V)**2, 0)
+            grossVQ = np.sum(abs(Sxx_V)**2, 0) / Den
+            grossHNR = 20*np.log10(abs((np.sum(abs(Sxx_V)**2, 0))/
+                                       (np.sum(abs(Sxx_U)**2, 0))))
+        elif meth.lower() == "rms":
+                nhop = nwin - novlp
+                Vbuff = buffer(V.signal, nwin, nhop)
+                Ubuff = buffer(U.signal, nwin, nhop)
+                nFrame = Vbuff.shape[1]
+                txx = np.arange(0,len(self.signal) - 1, nhop) / sr
+                txx = txx[:nFrame] + nwin / sr / 2
+        
+                grossVQ = np.zeros(nFrame)
+                grossHNR = np.zeros(nFrame)
+                w = scipy.signal.hann(nwin)
+        
+                for k in range(nFrame):
+                    xpt = Vbuff[:,k] * w 
+                    xpt = xpt - np.mean(xpt)
+                    xnt = Ubuff[:,k] * w 
+                    xnt = xnt - np.mean(xnt)
+                    grossVQ[k] = ((np.linalg.norm(xpt,2)**2)/
+                                  (np.linalg.norm(xpt,2)**2+np.linalg.norm(xnt,2)**2))**2
+                    grossHNR[k] = ((np.linalg.norm(xpt,2)**2)/(np.linalg.norm(xnt,2)**2))**2
+        else:
+             raise ValueError("The method to compute VQ does not exist. Please choose either spectro or rms")
+
+        VQ = lininterp1(txx, grossVQ, tvec) * 100
+        VQ[VQ < 0] = 0
+        VQ[VQ > 100] = 100
+        HNR = lininterp1(txx, grossHNR, tvec)
+        
+        self.voicing_quotient = VQ
+        self.hnr = HNR
+        
+        if self.time_vector is None:
+            self.time_vector = tvec
+            
+        return VQ, HNR
 class SpeechSegment(SpeechAudio):
 
     phoneme_label = None
